require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/personal_role/personal_role_get_territory_list'
require 'uu/os/personal_role/profile_attributes'
require 'uu/os/personal_role/territory_attributes'
require 'uu/os/personal_role/profile_set_attributes'
require 'uu/os/personal_role/territory_type'
require 'uu/os/personal_role/personal_role_set_artifact_presentation_attributes'
require 'uu/os/personal_role/personal_role_artifact_presentation_attributes'
require 'uu/os/personal_role/personal_role_portlets_mode'
require 'uu/os/personal_role/personal_role_portlet'
require 'uu/os/personal_role/personal_role_create'
require 'uu/os/personal_role/personal_role_set_attributes'
require 'uu/os/personal_role/personal_role_set_access_codes'
require 'uu/os/personal_role/personal_role_relocate'
require 'uu/os/personal_role/personal_role_attributes'
require 'json'

module UU
  module OS
    # Personal Role service.
    module PersonalRole
      extend self

      # Service path
      PATH = 'ues/core/personalrole/UESPersonalRole'
      # Creates a new Personal Role to a specified location.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of a folder or an organizational unit where a Personal Role will be created to.
      # @param [PersonalRoleCreate] personal_role Object containing attributes of the new Personal Role.
      # @option personal_role [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the Personal Role meta artifact. If the meta artifact is not specified or the user is not authorized to create a Personal Role according to the selected meta artifact, the command fails. Must not be nil.
      # @option personal_role [String] :first_name First name of the user represented by the Personal Role. When nil is set, the first name is set to the generated Universe ID.
      # @option personal_role [String] :surname Surname of the user represented by the Personal Role. When nil is set, the surname is set to “uuPerson”.
      # @option personal_role [String] :access_code1 Access code 1 of the Personal Role. Access code 1 must be 4-250 characters long and can only contain these characters: a-z, A-Z, 0-9, '.', '-', '_', '@'. The access code 1 must be unique in the Universe, otherwise the command fails. When nil is set, a random code is generated.
      # @option personal_role [String] :access_code2 Access code 2 of the Personal Role. Access code 2 must be 8-250 characters long and can only contain these characters: any character. When nil is set, a random code is generated.
      # @option personal_role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies whether the Personal Role will be cast into All roles from organizational unit. When nil is set, the value is set to true.
      # @option personal_role [Boolean] :searchable_in_territory Specifies whether the Personal Role will be searchable within the territory. When nil is set, the value is set to the default value from the meta artifact.
      # @option personal_role [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the Personal Role. When nil is set, the security clearance is set to NO_CONFIDENTIALITY.
      # @option personal_role [UU::OS::Artifact::SecurityLevel] :security_level Security level of the Personal Role. When nil is set, the security level is set to the default value from the meta artifact. If the default security level is higher than the user security clearance, the command fails.
      # @option personal_role [String] :code Code of the Personal Role. The code must be unique within the territory, otherwise the command fails. When nil is set, the code is set to the generated Universe ID.
      # @option personal_role [String] :description Description of the Personal Role.
      # @option personal_role [String, UU::OS::UESURI] :competent_role_uri UESURI of the Personal Role competent role. When nil is set, the executive role from authorization is used. If no role is authorized to create a Personal Role into the selected location, the command fails. If the selected role does not have a connected interface with the selected meta artifact, the command fails. If the selected role does not belong to the same organizational unit as the new Personal Role or to a superior organizational unit, the command fails.
      # @option personal_role [Boolean] :component_content Defines whether the primarily displayed content of the Personal Role is component content. When nil is set, the value is set to the default value from the meta artifact.
      # @option personal_role [String, UU::OS::UESURI] :icon_uri UESURI of the Personal Role icon. When nil is set, the Personal Role uses the default icon from the meta artifact. Allowed format of icon URI: ‘ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}’. An example of icon URI: ‘ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}’.
      # @option personal_role [String] :avatar_uri UESURI or URL of image which represents the Personal Role avatar (user photo). When nil is set, the Personal Role is created without an avatar.
      # @option personal_role [String] :universe_id Universe identifier of the user. When nil is set, universe identifier will be generated.
      # @option personal_role [Boolean] :external_access_code1_validation determines whether to validation access code 1 using an external service (the default value is true)
      #
      #
      # @return [UU::OS::UESURI] UESURI of the new personal role.
      #
      # @example
      #   # Create personalRole.
      #   UU::OS::PersonalRole.create("ues:MT.SMITH.JOHN:PERSONAL",
      #     :meta_artifact_uri => "ues:MT.SMITH.JOHN:UU.OS/PERSONAL_TERRITORY/PERSONAL_ROLE",
      #     :first_name => "John",
      #     :surname => "Smith",
      #     :access_code1 => "accessCode1",
      #     :access_code2 => "accessCode2",
      #     :searchable_in_territory=> true,
      #     :grant_all_from_organizational_unit=>true,
      #     :security_clearance=>'NO_CONFIDENTIALITY',
      #     :security_level=>'NO_CONFIDENTIALITY',
      #     :code => nil,
      #     :description => nil,
      #     :competent_role_uri => nil,
      #     :component_content => nil,
      #     :icon_uri => nil,
      #     :avatar_uri => nil,
      #     :external_access_code1_validation => true
      #   )
      #
      # @example
      #   # Creates a Personal Role into a Business Territory for users without their own Personal Territory,
      #   # in which case it should not be granted all from organizational unit and should not be searchable in territory.
      #   # First name and surname can be set later when the Personal Role is assigned to a specific user.
      #   UU::OS::PersonalRole.create("ues:PEOPLE-BT:ENV.ACRS/PACRS",
      #     :meta_artifact_uri => "ues:PEOPLE-BT:UU.OS/PERSONAL_TERRITORY/PERSONAL_ROLE",
      #     :grant_all_from_organizational_unit => false,
      #     :searchable_in_territory => false
      #   )
      #
      def create(location_uri, personal_role = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        payload = UU::OS::PersonalRole::PersonalRoleCreate.new(personal_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Sets attributes of a Personal Role.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri UESURI of a Personal Role whose attributes will be modified.
      # @param [PersonalRoleSetAttributes] personal_role Object containing modified attributes of the Personal Role.
      # @option personal_role [String] :first_name First name of the user represented by the Personal Role. When nil is set, the first name is not modified.
      # @option personal_role [String] :surname Surname of the user represented by the Personal Role. When nil is set, the surname is not modified.
      # @option personal_role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies whether the Personal Role is cast into All roles from organizational unit. When nil is set, the value is not modified.
      # @option personal_role [Boolean] :searchable_in_territory Specifies whether the Personal Role is searchable within the territory. When nil is set, the value is not modified.
      # @option personal_role [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the Personal Role. When nil is set, the security clearance is not modified.
      # @option personal_role [UU::OS::Artifact::SecurityLevel] :security_level Security level of the Personal Role. When nil is set, the security level is not modified.
      # @option personal_role [String] :code Code of the Personal Role. When nil is set, the code is not modified. When an empty string is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the command fails.
      # @option personal_role [String] :description Description of the Personal Role. When nil is set, the description is not modified.
      # @option personal_role [Boolean] :component_content Defines whether the primarily displayed content of the Personal Role is component content. When nil is set, the value is not modified.
      # @option personal_role [String, UU::OS::UESURI] :icon_uri UESURI of the Personal Role icon. When nil is set, the icon is not modified. When nil UESURI (“ues:[-1]:[-1]:”) is set, the Personal Role uses the default icon from the meta artifact. Allowed format of icon URI: ‘ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}’. An example of icon URI: ‘ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}’.
      # @option personal_role [String] :avatar_uri UESURI or URL of image which represents Personal Role avatar (user photo). When nil is set, the avatar is not modified. When an empty string is set, the value is erased.
      #
      # @return [UU::OS::UESURI] UESURI of the personal role.
      #
      # @example
      #   # Set attributes of a Personal Role.
      #   UU::OS::PersonalRole.set_attributes("ues:MT.SMITH.JOHN:6-6666-1",
      #     :first_name => "John",
      #     :surname => "Smith",
      #     :grant_all_from_organizational_unit => true,
      #     :searchable_in_territory => true,
      #     :security_clearance => "NO_CONFIDENTIALITY",
      #     :security_level => "NO_CONFIDENTIALITY",
      #     :code => "6-6666-1",
      #     :description => "",
      #     :component_content => true,
      #     :icon_uri => "ues:[-1]:[-1]:",
      #     :avatar_uri => ""
      #   )
      #
      def set_attributes(personal_role_uri, personal_role = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        payload = UU::OS::PersonalRole::PersonalRoleSetAttributes.new(personal_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', personal_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns attributes of a Personal Role.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri UESURI of a Personal Role whose attributes will be returned.
      #
      # @return [PersonalRoleAttributes] Object containing attributes of the specified Personal Role.
      #
      # @example
      #   # Returns attributes a Personal Role.
      #   UU::OS::PersonalRole.get_attributes("ues:MT.SMITH.JOHN:6-6666-1")
      #
      def get_attributes(personal_role_uri)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', personal_role_uri)
          return PersonalRole::PersonalRoleAttributes.new(res)
        end
      end

      # Returns a list of territories the personal role has access to. The command does not return
      # all territories, only a sublist. The list of returned territories can be filtered by the name, code or
      # type of a territory (See {UU::OS::PersonalRole::TerritoryType} for supported types). The list is sorted by names (and codes in case two names are equal) by default, or
      # it can be sorted arbitrarily.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri URI of the personal role whose territories are listed.
      # @param [PersonalRole::PersonalRoleGetTerritoryList] criteria DTO containing criteria for listing territories.
      #
      # @return [UU::OS::REST::ResultList<PersonalRole::PersonalRoleGetTerritoryList, PersonalRole::TerritoryAttributes>] List of DTO representing territories.
      #
      # @example
      #   # Get territory list for specified personal role.
      #   UU::OS::PersonalRole.get_territory_list('ues:MY_TERRITORY:PERSONAL_ROLE')
      #
      def get_territory_list(personal_role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        dto = UU::OS::PersonalRole::PersonalRoleGetTerritoryList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getTerritoryList', personal_role_uri)
          return UU::OS::REST::ResultList.new(PersonalRole::PersonalRoleGetTerritoryList, PersonalRole::TerritoryAttributes, res)
        end
      end

      # Returns profile information of the user.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri URI of the personal role whose profile attributes are returned.
      #
      # @return [PersonalRole::ProfileAttributes] ProfileAttributes DTO representing user's profile data.
      #
      # @example
      #   # Get profile attributes for certain role.
      #   UU::OS::PersonalRole.get_profile_attributes('ues:MY_TERRITORY:PERSONAL_ROLE')
      #
      def get_profile_attributes(personal_role_uri)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getProfileAttributes', personal_role_uri)
          return PersonalRole::ProfileAttributes.new(res)
        end
      end

      # Sets user's profile attributes.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri <b>Not nil.</b> URI of the personal role whose profile attributes are set.
      # @param [UU::OS::PersonalRole::ProfileSetAttributes] options Specific options.
      # @option options [String, UU::OS::PersonalRole::PersonalRoleLanguage] :language User language. The language is in the ISO 639-1 format. Changes may take effect with delay due to profile cache.
      # @option options [String] :notifications_email E-mail address for notifications. Must be specified if notifications are not turned off. For example email@email.com.
      # @option options [String, UU::OS::PersonalRole::PersonalRoleNotificationsRule] :notifications_rule Notification rule setting. If notifications are not turned off, notifications e-mail must be specified.
      # @option options [Boolean] :experimental_features_enabled Experimental features setting. Enables or disables the use of experimental features for the personal role. Changes may take effect with delay due to profile cache.
      # @option options [String, UU::OS::UESURI] :bookmarks_location_uri Location of user bookmarks. The location must be a Folder or Organizational Unit of the same territory as the Personal Role. Also, the Personal Role must have rights to insert into this location. If set to an empty string, no location will be set. If set to nil, the value will not be modified.
      # @option options [Boolean] :global_login_enabled Specifies whether the user global login is enabled. If set to nil, the value will not be modified.
      # @option options [Array] :portlets List of portlets displayed in the portlet area. The value is an array of portlet codes defined in {UU::OS::PersonalRole::PersonalRolePortlet}. An empty position with no portlet is specified by a nil value (or an empty string). Example: ["NAVIGATION", "NEW_ACTIVITIES", nil, "TABLE_OF_CONTENTS"]. The maximum allowed number of portlets (array size) is 5. If set to nil, the value will not be modified.
      # @option options [UU::OS::PersonalRole::PersonalRolePortletsMode] :portlets_mode Specifies whether the portlet area is displayed. Only VISIBLE or HIDDEN can be set. If set to nil, the value will not be modified.
      # @option options [UU::OS::PersonalRole::PersonalRolePortletsMode] :portlets_mobile_mode Specifies whether the portlet area is displayed on mobile devices. If set to DEFAULT, then the portlet area will be displayed on mobile devices according to the portlets_mode. If set to nil, the value will not be modified.
      # @option options [Boolean] :responsive_layout_enabled Responsive layout settings. Enables or disables using layout customized for devices with different screen sizes.
      # @option options [Boolean] :responsive_content_enabled Responsive content settings. Enables or disables showing artifact content customized for devices with different screen sizes.
      # @option options [Boolean] :bookmarks_to_navigator Enables or disables the automatic saving of links to artifacts in the navigator. Setting this attribute may not have any effect if the UC for adding bookmark to navigator is not available for given personal role.
      # @option options [String, UU::OS::UESURI] :preferred_role_uri UESURI of the preferred territory. It is the preferred territory of the user identified by the personal role. If nil is set, preferred territory will not be changed for the personal role.
      # @option options [String, UU::OS::UESURI] :preferred_territory_uri UESURI of the preferred role in the preferred territory. Those are the preferred territory and the preferred role of the user identified by the personal role. If nil is set, preferred role in the preferred territory will not be changed for the personal role. Preferred role can't be set if parameter preferred_territory_uri is nil.
      #
      # @return [UU::OS::UESURI] URI of the personal role, for which was the language settings modified.
      #
      # @example
      #   # Set the personals role language.
      #   UU::OS::PersonalRole.set_profile_attributes('ues:MY_TERRITORY:PERSONAL_ROLE', :language => 'en')
      #
      #   # Enable usage of experimental functionalities
      #   UU::OS::PersonalRole.set_profile_attributes('ues:MY_TERRITORY:PERSONAL_ROLE',
      #   :experimental_features_enabled => true
      #   )
      #
      def set_profile_attributes(personal_role_uri, options)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        attributesJson = UU::OS::PersonalRole::ProfileSetAttributes.new(options).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setProfileAttributes', personal_role_uri, attributesJson)
          return UU::OS::UESURI.new(res)
        end
      end

      # Configures user settings of artifact presentation. User will see artifacts according to this configuration. Presentation of artifact
      # can be affected by these attributes: artifact width, header mode (two lines, one line, none), control bar visibility, artifact header
      # sticking, artifact display mode (advanced, simple). When viewing an artifact, presentation settings are looked up in the following order:
      # User settings, artifact presentation settings, meta artifact template presentation settings, and layout artifact, if it is configured,
      # or default system settings otherwise.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         #
      #
      # @param [String, UU::OS::UESURI] personal_role_uri URI of the personal role whose profile attributes are to be configured.
      # @param [PersonalRoleSetArtifactPresentationAttributes] presentation_attributes DTO of presentation attributes
      # @option presentation_attributes [String, UU::OS::UESURI] :personal_role_uri Normalized UESURI of the personal role. Never nil.
      # @option presentation_attributes [UU::OS::Artifact::ArtifactWidthType] :artifact_width Width of the artifact. Never nil.
      # @option presentation_attributes [UU::OS::Artifact::ArtifactHeaderDisplayMode] :header_display_mode  Header type. Never nil.
      # @option presentation_attributes [UU::OS::Artifact::ArtifactControlBarDisplayMode] :control_bar_display_mode Show/hide control bar. Never nil.
      # @option presentation_attributes [Boolean] :pin_artifact_header Setting of the artifact header sticking to screen edge when scrolling. Never nil.
      # @option presentation_attributes [UU::OS::Artifact::ArtifactDisplayMode] :artifact_display_mode Show simple or advanced context menu. Never nil.
      #
      # @return [UU::OS::UESURI] URI of personal role
      #
      # @example
      #   # Set user presentation attributes of an artifact
      #   UU::OS::PersonalRole.set_artifact_presentation_attributes('ues:PERSONAL_TERRITORY:PERSONAL_ROLE',
      #      :artifact_width => UU::OS::Artifact::ArtifactWidthType::DEFAULT,
      #      :header_display_mode => UU::OS::Artifact::ArtifactHeaderDisplayMode::HIDDEN,
      #      :pin_artifact_header => true,
      #      :artifact_display_mode => UU::OS::Artifact::ArtifactDisplayMode::ADVANCED)
      #
      def set_artifact_presentation_attributes(personal_role_uri, presentation_attributes = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        payload = UU::OS::PersonalRole::PersonalRoleSetArtifactPresentationAttributes.new(presentation_attributes).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setArtifactPresentationAttributes', personal_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Retrieves user settings of artifact presentation. User will see artifacts according to this configuration. Presentation of artifact can
      # be affected by these attributes: artifact width, header mode (two lines, one line, none), control bar visibility, artifact header
      # sticking, artifact display mode (advanced, simple). When viewing an artifact, presentation settings are looked up in the following order:
      # User settings, artifact presentation settings, meta artifact template presentation settings, and layout artifact, if it is configured,
      # or default system settings otherwise.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri URI of the personal role whose profile attributes are to be retrieved.
      #
      # @return [PersonalRoleSetArtifactPresentationAttributes]  DTO containing user settings of artifact presentation
      #
      # @example
      #   # Get user presentation attributes of an artifact
      #   UU::OS::PersonalRole.get_artifact_presentation_attributes('ues:PERSONAL_TERRITORY:PERSONAL_ROLE')
      #
      def get_artifact_presentation_attributes(personal_role_uri)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getArtifactPresentationAttributes', personal_role_uri)
          return PersonalRole::PersonalRoleArtifactPresentationAttributes.new(res)
        end
      end

      # Relocates a Personal Role between two territories within the same Territory Container. It is not possible to relocate a Personal Role within the same territory or across Territory Containers. To relocate a Personal Role, it is necessary to have permissions to move the selected Personal Role and insert it into the selected location according to the selected meta artifact. Note that attributes related to authorization and structure are not preserved when relocating the Personal Role and their value is evaluated the same way as when creating a new Personal Role.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri UESURI of a Personal Role to relocate.
      # @param [PersonalRoleRelocate] personal_role Object containing a new location and modified attributes of the Personal Role.
      # @option personal_role [String, UU::OS::UESURI] :location_uri UESURI of a folder or an organizational unit in the target territory where the Personal Role will be relocated to. Must not be nil.
      # @option personal_role [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the Personal Role meta artifact in the target territory. If the meta artifact is not specified or the user is not authorized to create a Personal Role according to the selected meta artifact, the command fails. Must not be nil.
      # @option personal_role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies whether the Personal Role will be cast into All roles from organizational unit. When nil is set, the value is set to true.
      # @option personal_role [Boolean] :searchable_in_territory Specifies whether the Personal Role will be searchable within the territory. When nil is set, the value is set to the default value from the meta artifact.
      # @option personal_role [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the Personal Role. When nil is set, the security clearance is set to NO_CONFIDENTIALITY.
      # @option personal_role [UU::OS::Artifact::SecurityLevel] :security_level Security level of the Personal Role. When nil is set, the security level is set to the default value from the meta artifact. If the default security level is higher than the user security clearance, the command fails.
      # @option personal_role [String] :code Code of the Personal Role. The code must be unique within the territory, otherwise the command fails. When nil is set, the code is set to the generated Universe ID.
      # @option personal_role [String, UU::OS::UESURI] :competent_role_uri UESURI of the Personal Role competent role in the target territory. When nil is set, the executive role from authorization is used. If no role is authorized to create a Personal Role into the selected location, the command fails. If the selected role does not have a connected interface with the selected meta artifact, the command fails. If the selected role does not belong to the same organizational unit as the new Personal Role or to a superior organizational unit, the command fails.
      #
      # @return [UU::OS::UESURI] UESURI of the relocated Personal Role.
      #
      # @example
      #   # Relocates a Personal Role from an Access Territory to a Personal Territory.
      #    UU::OS::PersonalRole.relocate("ues:ACCESS-BT:6-6666-1",
      #      :location_uri => "ues:MT.SMITH.JOHN:PERSONAL",
      #      :meta_artifact_uri => "ues:MT.SMITH.JOHN:UU.OS/PERSONAL_TERRITORY/PERSONAL_ROLE",
      #      :grant_all_from_organizational_unit => true,
      #      :searchable_in_territory => true,
      #      :security_clearance => "NO_CONFIDENTIALITY",
      #      :security_level => "NO_CONFIDENTIALITY",
      #      :code => nil,
      #      :competent_role_uri => nil
      #    )

      def relocate(personal_role_uri, personal_role = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        payload = UU::OS::PersonalRole::PersonalRoleRelocate.new(personal_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('relocate', personal_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns a list of accounts of a Personal Role. The selected Personal Role must belong to the currently logged in user.
      #
      # @param[String, UU::OS::UESURI] personal_role_uri  URI of the Personal role whose accounts will be returned. Not nil.
      #
      # @return [PersonalRoleAccountAttributes] account_attributes Array of DTO with Personal role account attributes.
      # @option account_attributes [String] :id ID of the account. Never nil.
      # @option account_attributes [String] :realm Realm of the account.
      # @option account_attributes [String] :account_name Name of the account.
      # @option account_attributes [Date] :creation_time Time when the account was created for the Personal Role.
      #
      # @example
      #     UU::OS::PersonalRole.get_account_list('ues:PERSONAL_TERRITORY:PERSONAL_ROLE')
      #
      def get_account_list(personal_role_uri)
        result = nil;

        svc = UU::OS::REST::RemoteClient.new(PersonalRole);
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAccountList', personal_role_uri);
          if (!res.nil?)
            res = JSON.parse(res, :symbolize_names => true)
            if ( res != nil )
              result = Array.new();
              res.each do |item|
                result << PersonalRole::PersonalRoleAccountAttributes.new(item);
              end
            end
          end
        end

        return result;
      end

      # Removes an account of a Personal Role. Only an OAuth account, Mobile login account, One time login, Secure key login and Diary access account can be removed.
      # See {UU::OS::PersonalRole::PersonalRoleAccountRealm} for supported realms. The selected Personal Role must belong to the currently logged in user.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri URI of the Personal role whose account will be removed. Not nil.
      # @param [String, UU::OS::UESURI] account_attributes Object containing attributes of the account which will be removed. Not nil.
      # @option account_attributes [String] :id ID of the account. Not nil.
      #
      # @example
      #     UU::OS::PersonalRole.remove_account('ues:PERSONAL_TERRITORY:PERSONAL_ROLE', :id => '123456')
      def remove_account(personal_role_uri, account_attributes)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        payload = UU::OS::PersonalRole::PersonalRoleRemoveAccount.new(account_attributes).to_json;

        svc = UU::OS::REST::RemoteClient.new(PersonalRole);
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('removeAccount', personal_role_uri, payload);
        end
      end

      # Sets access codes of a Personal Role. It is possible to change both access codes or just one of them. For security reasons, both current access codes must be entered.
      #
      # @param [String, UU::OS::UESURI] personal_role_uri UESURI of a Personal Role whose attributes will be modified.
      # @param [PersonalRoleSetAccessCodes] access_codes Object containing both current and new access codes of the Personal Role.
      # @option access_codes [Boolean] :external_access_code1_validation determines whether to validation access code 1 using an external service (the default value is true)
      #
      # @return [UU::OS::UESURI] UESURI of the specified Personal Role.
      #
      # @example
      #   # Changes both access codes.
      #   UU::OS::PersonalRole.set_access_codes("ues:MT.SMITH.JOHN:6-6666-1",
      #     :access_code1 => "accessCode1",
      #     :access_code2 => "accessCode2",
      #     :new_access_code1 => "newAccessCode1",
      #     :new_access_code2 => "newAccessCode2",
      #     :external_access_code1_validation => true
      #   )
      #
      # @example
      #   # Changes access code 2 only.
      #   UU::OS::PersonalRole.set_access_codes("ues:MT.SMITH.JOHN:6-6666-1",
      #     :access_code1 => "accessCode1",
      #     :access_code2 => "accessCode2",
      #     :new_access_code2 => "newAccessCode2"
      #   )
      #
      def set_access_codes(personal_role_uri, access_codes = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalRole)
        payload = UU::OS::PersonalRole::PersonalRoleSetAccessCodes.new(access_codes).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAccessCodes', personal_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

    end
  end
end
