require 'uu/os/qos/qos_handler'
require 'uu/os/rest/binary_value'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'date'
require 'uu/os/property_template/property_template_attributes'
require 'uu/os/property_template/property_template_create'
require 'uu/os/property_template/property_template_set_attributes'
require 'uu/os/property_template/property_template_collection_attributes'
require 'uu/os/property_template/property_template_collection_create'
require 'uu/os/property_template/property_template_set_collection_attributes'
require 'uu/os/property_template/property_template_delete_collection'
require 'uu/os/property_template/property_template_get_entry_list'
require 'uu/os/property_template/property_template_entry_attributes'
require 'uu/os/property_template/property_template_move'
require 'uu/os/property_template/property_template_move_collection'
require 'uu/os/attachment/stream_wrapper'

module UU
  module OS
    # UES Property Template service.
    module PropertyTemplate
      extend self

      # Service path
      PATH = 'ues/core/property/UESPropertyTemplate'
      # Returns attributes representing the specified property template.
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of of the property template
      #
      # @return [PropertyTemplateAttributes] Object with property template attributes
      #
      # @example      
      #   #Get attributes of a property template 
      #   UU::OS::PropertyTemplate.get_attributes('ues:TERRITORY:META_ARTIFACT:PROPERTY_TEMPLATE')
      def get_attributes(template_uri)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', template_uri)
          return PropertyTemplate::PropertyTemplateAttributes.new(res)
        end
      end

      # Sets a new value on the specified property template. The data type of the value must match the data type of the specified
      # property template.
      #
      # @param [String, UU::OS::UESURI] template_uri UES URI of property template
      # @param [Object] value the new value of the property template. Possible types of property's value (there may be additional
      #   business restrictions based on property type, see {UU::OS::Property::PropertyType} for possible types      
      #
      # @example      
      #   #Set value of a property template of binary type
      #   UU::OS::PropertyTemplate.set_value('ues:TERRITORY:META_ARTIFACT:PROPERTY_TEMPLATE_BLOB', 
      #     UU::OS::REST::BinaryValue.new(data:'example', name:'my_name.txt'))
      #
      #   #Set value of a property template of text type
      #   UU::OS::PropertyTemplate.set_value('ues:TERRITORY:META_ARTIFACT:PROPERTY_TEMPLATE_TEXT', 'some string')
      def set_value(template_uri, value = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        payload = {}
        
        if (value.kind_of?IO)
          value = UU::OS::REST::BinaryValue.new(value)
        end
        if ((value.kind_of?UU::OS::REST::BinaryValue) && (value.data.nil?) && (!value.name.nil? && !value.name.empty?))
          payload[:defaultName] = value.name
        elsif (value.kind_of?UU::OS::REST::BinaryValue) && (!value.data.nil?)
          if (value.data.kind_of?File) && (File.basename(value.data.path) == value.name)
            payload[:defaultName] = value.name
          end
          if (value.data.kind_of?File)
            payload[:value] = create_file_with_file_name(value)
          else
            payload[:value] = create_stream_with_file_name(value)
          end
        else
          payload = value.to_json
        end
          
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setValue', template_uri, payload)
        end
      end

      # Returns the value of the given property template of the corresponding type. Null in case the value is not set. 
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the property template
      #
      # @return [Object] the value of the property template
      #
      # @example      
      #   #Get value of the property template
      #   value = UU::OS::PropertyTemplate.get_value('ues:TERRITORY:META_ARTIFACT:PROPERTY_TEMPLATE')
      def get_value(template_uri)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          raw = svc.raw_get_binary('getValue', template_uri)
          type = raw[2][:uu_property_value_type]
          value = svc.process_result(raw)
          result = deserialize_by_type(value,type)
          if (result.kind_of?UU::OS::REST::BinaryValue)
            if (raw[2][:content_disposition] != nil)
              disps = raw[2][:content_disposition].split(";")
              disps.each do |part|
                kv = part.lstrip.split("=")
                if kv[0]=="filename"
                  result.name = kv[1]
                end
              end
            end
          end
          return result
        end
      end
      
      # Returns a list of entries (property templates or property template collections) enlisted in the specified property
      # template collection or on the specified Meta Artifact (property template entries from the Meta Artifact root property template
      # collection are returned in that case). Command does not return all entries, only a sublist. The list of returned entries could be
      # filtered by the name, code or type of an entry. The list is sorted by the name (and code in case are equal) of an entry by default, or
      # it can be sorted arbitrarily by any of entry attributes usable for the filtering or these attributes combinations. See the UES query
      # documentation for more information.
      #
      # @param [String, UU::OS::UESURI] parent_node_uri UESURI of the Meta Artifact or of the property template collection from where entries (property templates or
      #          property template collections) are listed.      
      # @param [PropertyTemplateGetEntryList] criteria criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list (more information in {uuQuery - Guideline}[https://plus4u.net/ues/sesm?SessFree=ues%253AUNI-BT%255B210795%255D%253AUES.DOC%252FCMD_V1%252FUESQUERY%255B44191574519623267%255D%253A]).
      # It is possible to filter result by: {PropertyTemplateEntryAttributes#name name},
      # {PropertyTemplateEntryAttributes#code code}, {PropertyTemplateEntryAttributes#entity_type_uri entityTypeUri},
      # {PropertyTemplateEntryAttributes#value_type valueType}
      #
      # @return [Array] An Array with the command result. Contains instances of {UU::OS::PropertyTemplate::PropertyTemplateEntryAttributes}.
      #
      # @example      
      #   #Get list of entries
      #   list = UU::OS::PropertyTemplate.get_entry_list('ues:TERRITORY:META_ARTIFACT')
      #   #Write property template values
      #   list.each { |e| puts e.value }
      def get_entry_list(parent_node_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        dto = PropertyTemplate::PropertyTemplateGetEntryList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)        
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getEntryList', parent_node_uri)
          res = UU::OS::REST::ResultList.new(PropertyTemplate::PropertyTemplateGetEntryList, PropertyTemplate::PropertyTemplateEntryAttributes, res)
          return res
        end
      end

      # Creates a new property template on the specified Meta Artifact or in the specified property template group. At the very
      # least property template type and name have to be specified in this command. Value of the property template is optional.
      #
      # @param [String, UU::OS::UESURI] parent_node_uri of the Meta Artifact or of the property template group where the new property template is created
      # @param [PropertyTemplateCreate] property_template Object containing attributes of the new property template
      # @option property_template [String] :name Not nil. Name of the new property template.
      # @option property_template [String] :code Code of the new property template. When null is set, the code is generated automatically using the system sequence. The code must be
      #     unique in a given Meta Artifact, otherwise the command fails.
      # @option property_template [String] :description Description of the new property template.
      # @option property_template [UU::OS::Property::PropertyType] :type Not nil. Specifies the type of the new property template. 
      # @option property_template [Object] :value The value of the new property template. Passed object type has to correspond to the type of the new property template,
      #   see {UU::OS::Property::PropertyType} for possible types
      # @option property_template [String] :value_restriction String defining value restriction, one of the following values are possible:
      #   string with UESURI - to define enumeration restriction,
      #   string with Pattern - to define regular expression restriction,
      #   string with Query in form "metaArtifactUri = '...' OR metaArtifactUri = '...' AND organizationalUnitUri = '...' OR organizationalUnitUri = '...' (organizationalUnitUri = ${this-ou})" - to define restriction on Meta Artifact and organizational unit
      #
      # @return [UU::OS::UESURI] UESURI of created property template.
      #
      # @example      
      #   #Create number with regular expression restriction 
      #   uri = UU::OS::PropertyTemplate.create('ues:TERRITORY:META_ARTIFACT', :name => 'number Property', :type => UU::OS::Property::PropertyType::NUMBER, 
      #       :value => 4, :value_restriction => '[0-9]')
      #   #Create text with enum restriction
      #   uri = UU::OS::PropertyTemplate.create('ues:TERRITORY:META_ARTIFACT', :name => 'text Property', :type => UU::OS::Property::PropertyType::TEXT, 
      #       :value => 'valueFromEnum', :value_restriction => 'ues:TERRITORY:META_ARTIFACT:ENUM')
      #   #Create reference with only mar restriction
      #   uri = UU::OS::PropertyTemplate.create('ues:TERRITORY:META_ARTIFACT', :name => 'ref Property', :type => UU::OS::Property::PropertyType::REFERENCE, 
      #       :value => 'ues:TERRITORY:ARTIFACT', :value_restriction => "metaArtifactUri = 'ues:TERRITORY:META_ARTIFACT'")
      #   #Create reference with mar and ou restriction
      #   uri = UU::OS::PropertyTemplate.create('ues:TERRITORY:META_ARTIFACT', :name => 'ref Property', :type => UU::OS::Property::PropertyType::REFERENCE, 
      #       :value_restriction => "metaArtifactUri = 'ues:TERRITORY:META_ARTIFACT' AND organizationalUnitUri = ${this-ou}")
      def create(parent_node_uri, property_template = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        prop = UU::OS::PropertyTemplate::PropertyTemplateCreate.new(property_template)
        value = prop.value
        
        if (value.kind_of?File)
          prop.value = UU::OS::REST::BinaryValue.new(value)
        end
        if (prop.value.kind_of?UU::OS::REST::BinaryValue)
          payload = prop.to_hash
          payload.delete(:value)
          if (prop.value.data.kind_of?File)
            payload[:value] = create_file_with_file_name(prop.value)
          else
            payload[:value] = create_stream_with_file_name(prop.value)
          end
        else
          payload = prop.to_json
        end
          
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', parent_node_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Deletes the property template from the system.
      #
      # @param [String, UU::OS::UESURI] template_uri of the property template which is about to be deleted
      #
      # @example      
      #   #Delete property template  
      #   UU::OS::PropertyTemplate.delete('ues:TERRITORY:META_ARTIFACT:PROPERTY_TEMPLATE')
      def delete(template_uri)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', template_uri)
        end
      end

      # Sets basic attributes of the specified property template.      
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the property template
      # @param [PropertyTemplateSetAttributes] property_template Object of the new attributes for the property template
      # @option property_template [String] :name The new name of the property template. When null is set, the name is not modified. If set, name cannot be blank.
      # @option property_template [String] :code The new code of the property template. When null is set, the code is not modified. The code must be unique in a given Meta Artifact, otherwise the command fails.
      # @option property_template [String] :description The new description of the property template. When null is set, the description is not modified.
      # @option property_template [String] :value_restriction String defining value restriction, one of the following values are possible:
      #   string with UESURI - to define enumeration restriction,
      #   string with Pattern - to define regular expression restriction,
      #   string with Query in form "metaArtifactUri = '...' OR metaArtifactUri = '...' AND organizationalUnitUri = '...' OR organizationalUnitUri = '...' (organizationalUnitUri = ${this-ou})" - to define restriction on meta-artifact and organizational unit,
      #   empty string - will remove existing restrictions from property template.
      #
      # @return [UU::OS::UESURI] UESURI of the property template with new attributes
      #
      # @example
      #   #PropertyTemplate uri  
      #   uri = 'ues:TERRITORY:META_ARTIFACT:PROPERTY_TEMPLATE'
      #   #Remove restrictions from text 
      #   uri = UU::OS::PropertyTemplate.set_attributes(uri, :value_restriction => '')
      #   #Remove value 
      #   uri = UU::OS::PropertyTemplate.set_value(uri, '')
      #   #Set new restrictions 
      #   uri = UU::OS::PropertyTemplate.set_attributes(uri, :value_restriction => '^[a-zA-Z]+$')
      def set_attributes(template_uri, property_template = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        payload = UU::OS::PropertyTemplate::PropertyTemplateSetAttributes.new(property_template).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', template_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end      

      # Creates a new property template collection on the specified Meta Artifact or in the specified property collection
      # template. At the very least property template collection name has to be specified in this command.
      #
      # @param [String, UU::OS::UESURI] parent_node_uri of the Meta Artifact or of the property collection template where the new property collection template is created
      # @param [PropertyTemplateCollectionCreate] collection Object containing attributes of a new property collection template
      # @option collection [String] :name Not nil. Name of the new property collection template. 
      # @option collection [String] :code Code of the property collection template. When null is set, the code is generated automatically using the system sequence. The code must be
      #    unique within a given Meta Artifact, otherwise the command fails.
      # @option collection [String] :description Description of the new property template collection. When null is set, the description is left empty.
      #
      # @return [UU::OS::UESURI] UESURI of the new property collection template
      #
      # @example
      #   #Create property template collection 
      #   uri = UU::OS::PropertyTemplate.create_collection('ues:TERRITORY:META_ARTIFACT', :name => 'new Collection', :code => 'COLL')   
      def create_collection(parent_node_uri, collection = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        payload = UU::OS::PropertyTemplate::PropertyTemplateCollectionCreate.new(collection).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('createCollection', parent_node_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns attributes of the specified property collection template.
      #
      # @param [String, UU::OS::UESURI] collection_uri of the property collection template, which attributes are to be retrieved.
      #
      # @return [PropertyTemplateCollectionAttributes] Object with attributes of the property collection template.
      #
      # @example
      #   #Get property template collection attributes
      #   attrs = UU::OS::PropertyTemplate.get_collection_attributes('ues:TERRITORY:META_ARTIFACT:COLLECTION')   
      def get_collection_attributes(collection_uri)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getCollectionAttributes', collection_uri)
          return PropertyTemplate::PropertyTemplateCollectionAttributes.new(res)
        end
      end

      # Modifies basic attributes of the property collection template specified in collTemplateUri.
      #
      # @param [String, UU::OS::UESURI] collection_uri UESURI of the property collection template which attributes is modified
      # @param [PropertyTemplateSetCollectionAttributes] collection Object containing attributes of the property collection template
      # @option collection [String] :name The new name of the property collection template. When null is set, the name is not modified. If set, name cannot be blank.
      # @option collection [String] :code The new code of the property collection template. When null is set, the code is not modified. 
      #   The code must be unique in a given Meta Artifact, otherwise the command fails.
      # @option collection [String] :description The new description of the property collection template. When null is set, the description is not modified.
      #
      # @return [UU::OS::UESURI] UESURI of the property collection template
      #
      # @example      
      #   #Set new attributes
      #   newUri = UU::OS::PropertyTemplate.set_collection_attributes('ues:TERRITORY:META_ARTIFACT:COLLECTION', :name => 'new Name', :code => 'NEW_COLL')
      def set_collection_attributes(collection_uri, collection = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        payload = UU::OS::PropertyTemplate::PropertyTemplateSetCollectionAttributes.new(collection).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setCollectionAttributes', collection_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end
      
      # Moves a property template to the new location (see {PropertyTemplateMove#parent_node_uri}) on the same artifact. New
      # location can be property template collection or Meta Artifact.
      #
      # @param [String, UU::OS::UESURI] template_uri UESURI of the property template to be moved.
      # @param [PropertyTemplateMove] collection Object containing the new location of the property template
      # @option collection [String] :parent_node_uri Not nil. UESURI of a parent node (Meta Artifact or property template collection) where the property template will be moved. Property
      #    template can be moved only in the same Meta Artifact. 
      #
      # @example      
      #   #Move property template 
      #   UU::OS::PropertyTemplate.move('ues:TERRITORY:META_ARTIFACT:TEMPLATE', :parent_node_uri => 'ues:TERRITORY:META_ARTIFACT:COLLECTION')
      def move(template_uri, collection = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        payload = UU::OS::PropertyTemplate::PropertyTemplateMove.new(collection).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('move', template_uri, payload)
        end
      end
      
      # Moves a property template collection to the new location (see {PropertyTemplateMoveCollection#parent_node_uri}) on the same Meta Artifact. This template collection can not be base template
      # collection. New location can be property template collection or Meta Artifact, and it can not be child of or the same as property
      # template collection which is about to be moved.
      #
      # @param [String, UU::OS::UESURI] collection_uri UESURI of the property template collection which is about to be moved.
      # @param [PropertyTemplateMoveCollection] collection Object containing the new location of the property template collection.
      # @option collection [String] :parent_node_uri Not nil. UESURI of Meta Artifact or Property Template Collection where the property template collection will be moved. Property
      #    template collection can be moved only in the same Meta Artifact. ParentNodeUri can not be child of the moved collection or the same. 
      #
      # @example      
      #   #Move property template collection
      #   UU::OS::PropertyTemplate.move_collection('ues:TERRITORY:META_ARTIFACT:COLLECTION', :parent_node_uri => 'ues:TERRITORY:META_ARTIFACT:COLLECTION')
      def move_collection(collection_uri, collection = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        payload = UU::OS::PropertyTemplate::PropertyTemplateMoveCollection.new(collection).to_json
  
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('moveCollection', collection_uri, payload)
        end
      end

      # Deletes the specified property collection template. When recursive is set in options, all nested objects in property collection
      # template are also removed.
      #
      # @param [String, UU::OS::UESURI] collection_uri UESURI of the property collection template which is about to be deleted
      # @param [PropertyTemplateDeleteCollection] collection Object containing options for deletion
      # @option collection [Boolean] :recursive Flag specifies if non-empty property collection template will be deleted. When property template collection is not empty, 
      #   and the flag is set to true, all the nested objects will be deleted with the collection template asynchronously. If property template collection is not empty, and the
      #   flag is set to false, or not set, the command will fail.
      #
      # @example      
      #   #Delete property template collection
      #   UU::OS::PropertyTemplate.delete_collection('ues:TERRITORY:META_ARTIFACT:COLLECTION')
      def delete_collection(collection_uri, collection = nil)
        svc = UU::OS::REST::RemoteClient.new(PropertyTemplate)
        payload = UU::OS::PropertyTemplate::PropertyTemplateDeleteCollection.new(collection).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('deleteCollection', collection_uri, payload)
        end
      end      

      private

      def deserialize_by_type(value,type)

        return nil unless value        
        result = nil
        case type
        when Property::PropertyType::BINARY, Property::PropertyType::IMAGE          
            result = UU::OS::REST::BinaryValue.new(value, true)          
        when Property::PropertyType::DATE
          if value != nil and value != ""
            v = value.read
            if v != nil and v != ""
              result = DateTime.strptime(eval(String.new("\"" + v + "\"")),'%Y-%m-%d')
            end
          end
        when Property::PropertyType::INTERTERRITORIAL_REFERENCE,Property::PropertyType::REFERENCE
          if value != nil and value !=""
            v = value.read
            if v != nil and v != ""
              result = UU::OS::UESURI.new(eval(String.new("\"" + v + "\"")))
            end
          end
        when Property::PropertyType::NUMBER
            result = eval(String.new("\"" + value.read + "\"")).to_f
        when Property::PropertyType::SHORT_TEXT, Property::PropertyType::TEXT, Property::PropertyType::URL
            result = String.new(value.read).force_encoding("UTF-8")
        end
        return result

      end
      
      def create_stream_with_file_name(bin_val)
        bin_val.data.rewind
        UU::OS::Attachment::StreamWrapper.new(bin_val.data, "#{bin_val.name}")
      end

      def create_file_with_file_name(bin_val)
        file = File.new(bin_val.data.path,'rb')
        path = bin_val.name.nil? || bin_val.name.strip.empty? ? file.path : "#{bin_val.name}"
        UU::OS::Attachment::StreamWrapper.new(file, path)
      end

    end
  end
end
