require 'uu/os/qos/qos_handler'
require 'uu/os/uesuri'
require 'uu/os/use_case/use_case_create'
require 'uu/os/use_case/use_case_attributes'
require 'uu/os/use_case/use_case_set_attributes'
require 'uu/os/use_case/use_case_set_options'
require 'uu/os/use_case/use_case_get_options'
require 'uu/os/use_case/use_case_get_use_case_list'
require 'base64'
require 'uu/os/rest'


module UU
  module OS
    module UseCase
      extend self

      # Service path
      PATH = 'uu/os/UseCase'
      # The command creates a use case in a specified meta artifact. @param metaArtifactUri
      # URI of the meta artifact where the use case is aggregated. @paramuseCase DTO containing
      # attributes of the new use case @return URI of the new use case @throws UseCaseRTException
      # 
      # @param [String, UU::OS::UESURI] meta_artifact_uri UESURI of MetaArtifact in which the UC will be created
      # @param [UseCaseCreate] usecase DTO containing attributes needed for UC creation
      # @option usecase [String] :name The name of the use case. Cannot be nil.
      # @option usecase [String] :code The code of the use case. The code must be unique in the given meta artifact, otherwise the create command fails.
      # @option usecase [String] :description The description of the use case. When nil is set, the description will be empty.   
      # @option usecase [String, UU::OS::UESURI] :use_case_realization_uri The UESURI of a use case realization. 
      # @option usecase [String, UU::OS::REST::BinaryValue] :options Options data for use case calling. Accepted format is JSON.   
      #
      # @return [UU::OS::UESURI] UESURI of the created UseCase
      # 
      # @example
      #   # Create usecase for metaartifact.
      #   UU::OS::UseCase.create('ues:TERRITORY:METAARTIFACT', :name=>'Usecase name', :code=>'USECASE', 
      #   :description=>'Some sample data', :use_case_realization_uri=>'ues:TERRITORY:USECASE_REALIZATION', :options=>'{"key1":"value1","key2":true}' )
      #
      def create(meta_artifact_uri, usecase = nil)
        svc = UU::OS::REST::RemoteClient.new(UseCase)
        payload = UU::OS::UseCase::UseCaseCreate.new(usecase).to_json               

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', meta_artifact_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # The command deletes the specified use case from the system. 
      # 
      # @param [String, UU::OS::UESURI] use_case_uri UESURI of the UC to be deleted
      #
      # @example
      #   # Delete usecase specified by ues uri
      #   UU::OS::UseCase.delete('ues:TERRITOTY:USECASE')
      #
      def delete(use_case_uri)
        svc = UU::OS::REST::RemoteClient.new(UseCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', use_case_uri)
        end
      end

      # This command returns attributes representing the use case specified by the useCaseUri
      # parameter.
      # of the use case
      # 
      # @param [String, UU::OS::UESURI] use_case_uri UESURI URI of the use case
      #
      # @return [UseCaseAttributes] DTO containing attributes
      #
      # @example
      #   # Get attributes of usecase.
      #   UU::OS::UseCase.get_attributes('ues:TERRITORY:USECASE')
      #
      def get_attributes(use_case_uri)
        svc = UU::OS::REST::RemoteClient.new(UseCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', use_case_uri)
          return UseCase::UseCaseAttributes.new(res)
        end
      end

      # The command sets attributes on specified use case according to useCaseUri.
      # 
      # @param [String, UU::OS::UESURI] use_case_uri UESURI  of the use case
      # @param [UseCaseSetAttributes] usecase DTO containing newattributes of the use case
      # @option usecase [String] :name The name of the use case.
      # @option usecase [String] :code The code of the use case.
      # @option usecase [String] :description The description of the use case.
      # @option usecase [String, UU::OS::UESURI] :use_case_realization_uri The UESURI of a use case realization.If it is not specified, the value is not modified.If nil URI ('ues:{[-1]}:{[-1]}:') is set, use case realization is detached. Detaching use case realization from use case is not recommended as it may lead to inaccessibility of certain features provided by this use case. Use case realization should either be in the same meta model as use case object or in another meta model with code VENDOR_CODE '.' APP_CODE '-V' VERSION.
      #
      # @return [UU::OS::UESURI] UESURI of the use case.
      #
      # @example
      #   # Set attributes of usecase.
      #   useCaseUri = UU::OS::UseCase.set_attributes('ues:TERRITORY:USECASE', :name=>'Visual usecase name', :code=>'NEW_CODE',
      #   :description=>'Some sample description', :use_case_realization_uri=>'ues:TERRITORY:USECASE_REALIZATION')
      #      
      def set_attributes(use_case_uri, usecase = nil)
        svc = UU::OS::REST::RemoteClient.new(UseCase)
        payload = UU::OS::UseCase::UseCaseSetAttributes.new(usecase).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', use_case_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # The command sets options of the specified use case.
      # 
      # @param [String, UU::OS::UESURI] uri UESURI of the use case
      # @param [UseCaseSetOptions] dto DTO containing the options data for setting use case options.
      # @option dto [String, UU::OS::REST::BinaryValue] :options Data for use case options. Only JSON ("application/json") format is accepted.
      # 
      # @example
      #   # Set options for usecase.
      #   UU::OS::UseCase.set_options('ues:TERRITORY:USECASE',:options=>'{"key1":"value1","key2":true}')
      #
      def set_options(uri, dto = nil)
        svc = UU::OS::REST::RemoteClient.new(UseCase)
        payload = UU::OS::UseCase::UseCaseSetOptions.new(dto).to_json
        
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('setOptions', uri, payload)
        end
      end

      # The command returns options of the specified use case.
      # 
      # @param [String, UU::OS::UESURI] use_case_uri UESURI of the use case 
      # @param [UseCaseGetOptions] get_dto DTO containing options for getting usecase options.
      # @option get_dto [Boolean] :summed When summed is true, the command returns summed options from the meta use case realization, use case realization and use case. Otherwise the command returns use case options only.
      # @option get_dto [String] :requested_mime_type Type of use case options. When nil is set, the JSON value is used.
      #
      # @return [UseCaseGetOptions] DTO containing the use case options in a stream.
      # 
      # @example
      #   # Get use case options from usecase only - default returned format will be json.
      #   UU::OS::UseCase.get_options('ues:TERRITORY:USECASE',:summed=>false)
      #   
      def get_options(use_case_uri, get_dto = nil)
        svc = UU::OS::REST::RemoteClient.new(UseCase)
        dto = UU::OS::UseCase::UseCaseGetOptions.new(get_dto)
        svc.add_parameter('requestedMimeType', dto.requested_mime_type)
        UU::OS::QoS::QoSHandler.auto_retry do
          ret = svc.get('getOptions', use_case_uri)
          return UU::OS::REST::BinaryValue.new(ret, true)
        end
      end

      # Returns list of the use cases that are defined on the specified meta artifact. The
      # list of returned use cases could be filtered by the name, code, useCaseRealizationUri and useCaseType.
      # Returned list does not contain any objects filtered out by executed UESQuery. The list is sorted by 
      # the name (and codes in case are equal) of an use case by default.
      # 
      # 
      # @param [String, UU::OS::UESURI] meta_artifact_uri UESURI of the meta artifact uri
      # @param [UseCaseGetUseCaseList] criteria DTO containing the criteria for listing use cases.
      # @option criteria [String] :query Query in uesQuery syntax (see documentation for more info). Records can be filtered and ordered by name, code, useCaseRealizationUri and useCaseType. If set to nil, no filtering will be applied and results will be ordered by name and code as the second criteria.
      #
      # @return [UU::OS::REST::ResultList<UseCase::UseCaseGetUseCaseList, UseCase::UseCaseAttributes>] List of DTO with attributes of use cases
      #
      # @example
      #   # Return usecases for specified meta artifact and descend ordered by code. 
      #   UU::OS::UseCase.get_use_case_list('ues:TERRITORY:METAARTIFACT', :criteria=>'ORDER BY code DESC')
      #
      def get_use_case_list(meta_artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(UseCase)
        dto = UseCase::UseCaseGetUseCaseList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getUseCaseList', meta_artifact_uri)
          return UU::OS::REST::ResultList.new(UseCase::UseCaseGetUseCaseList, UseCase::UseCaseAttributes, res)
        end
      end

      # The command returns a normalized URI of use case by a specified URI of action. <br>
      # <br> In example: <ul> <li>input:enitityUri="ues:UNI-BT:ART", actionUri="uu-uc:UNI-BT:ART?Property/create"
      # <li>output: "ues:UNI-BT:MAR:uu.os/Property/create" </ul>@param enitityUri
      # URI of a entity from which the use case is called @param options DTO containing the
      # options for getting use case URI @return URiof the found use case @throws UseCaseRTException
#      def get_use_case_uri(enitity_uri, usecase = nil)
#        svc = UU::OS::REST::RemoteClient.new(UseCase)
#        payload = UU::OS::UseCase::UseCaseGetUseCaseUri.new(usecase).to_json
#
#        UU::OS::QoS::QoSHandler.auto_retry do
#          res = svc.get('getUseCaseUri', enitity_uri, payload)
#          return UU::OS::UESURI.new(res)
#        end
#      end

    end
  end
end
