require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/uuee/uuee_attributes'
require 'uu/os/uuee/uuee_set_attributes'
require 'uu/os/uuee/uuee_create'
require 'uu/os/uuee/uuee_set_access_codes'

module UU
  module OS
    module UUEE
      extend self

      # Service path of the REST API.
      PATH = 'ues/core/role/UESUUEE'
      # This command returns attributes representing the uuEE specified
      # by the +uueeUri+ parameter. The command does not change the state of the role
      # in the system.
      #
      # @param [String, UU::OS::UESURI] uueeUri UESURI of the uuEE whose attributes will be returned.
      #
      # @return [UUEEAttributes] DTO containing the uuEE attributes
      #
      # @example
      #   # Get attributes of specified uuEE.
      #   UU::OS::UUEE.get_attributes('ues:TERRITORY:UUEE')
      #
      def get_attributes(uueeUri)
        svc = UU::OS::REST::RemoteClient.new(UUEE)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', uueeUri)
          return UUEE::UUEEAttributes.new(res)
        end
      end

      # Command for setting attributes of a uuEE. The command can't modify references
      # to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] uueeUri UESURI of the uuEE whose attributes will be modified.
      # @param [UUEESetAttributes] uuee Object containing the modified attributes of the uuEE.
      # @option uuee [String] :first_name The new first name of the uuEE. When nil is set, the first name is not modified.
      # @option uuee [String] :surname The new surname of the uuEE. When nil is set, the surname is not modified.
      # @option uuee [String] :code The new code of the uuEE. When nil is set, the code is not modified. When blank string is set, the code is generated automatically using the system sequence. The code must be unique in a given territory, otherwise the setAttributes command fails.
      # @option uuee [String] :description The new description of the uuEE. When nil is set, the description is not modified.
      # @option uuee [UU::OS::Artifact::SecurityLevel] :security_level Security level of the new uuEE. When nil is set, security level is taken from the meta artifact SecurityLevel. If the meta artifact SecurityLevel is higher than the user’s security clearance, the create command fails.
      # @option uuee [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the new uuEE. When nil is set, the security clearance is not modified.
      # @option uuee [Boolean] :searchable_in_territory Specifies if it is possible to search the uuEE in its territory. When nil is set, this attribute on the uuEE is not modified.
      # @option uuee [String, UU::OS::UESURI] :icon_uri URI of the uuEE’s icon. When nil is set, the icon is not modified. If nil UESURI (“ues:[-1]:[-1]:”) is set, the uuEE icon is removed and the uuEE uses icon from meta artifact. Allowed format of icon URI: ‘ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{CODE_OF_ICON}’. An example of icon URI: ‘ues:#{system}:#{ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{ART_077}’.
      # @option uuee [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the uuEE will be included among All roles from organizational unit. When nil is set, this attribute on the uuEE is not modified.
      # @option uuee [Boolean] :component_content Defines whether the primarily displayed content of the created uuEE is component content. Default value is false.
      # @option uuee [String] :app_code Full code of the application which is represented by the uuEE. The code must be in the format VENDOR_CODE.APP_CODE. If set to an empty string, the code will be erased (uuEE will not be an identity of any application). If set to nil, the value will not be modified.
      # @option uuee [String] :avatar_uri UESURI or URL of image which represents uuEE's avatar (user photo). If set to an empty string, the value will be erased. When nil is set, the value is not modified.
      #
      # @return [UU::OS::UESURI] UESURI of the uuEE
      #
      # @example
      #   #Set attributes of personal access role.
      #   uuEEUri = UU::OS::UUEE.set_attributes('ues:TERRITORY:UUEE',
      #                 :first_name=>'John',
      #                 :surname=>'Doe',
      #                 :code=>'NEWUUEE',
      #                 :description=>'uuEE',
      #                 :security_level=>UU::OS::Artifact::SecurityLevel::CONFIDENTIAL,
      #                 :searchable_in_territory=>false,
      #                 :app_code=>'CDS.GB')
      #
      def set_attributes(uueeUri, uuee = nil)
        svc = UU::OS::REST::RemoteClient.new(UUEE)
        payload = UU::OS::UUEE::UUEESetAttributes.new(uuee).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', uueeUri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Creates an uuEE to the specified location.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the folder/org. unit/meta model where the personal access role will be created.
      # @param [UUEECreate] uuee DTO containing the attributes of the uuEE.
      # @option uuee [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact. UUEE is created only by meta artifact with the template class of UUEE. If this meta artifact will not be specified or if the rights does not permit to create a new uuEE from this meta artifact, the create command fails.
      # @option uuee [String] :app_code Full code of the application which be represented by the uuEE. The code must be in the format VENDOR_CODE.APP_CODE. If set to an empty string or nil, the uuEE will not be an identity of any application.
      # @option uuee [String] :first_name The first name of the uuEE. When nil is set, automatically will be set to the value of the uuEE universe ID.
      # @option uuee [String] :surname The surname of the uuEE. When nil is set, automatically will be set to "uuEE".
      # @option uuee [String] :code The code of the uuEE. The code must be unique in a given territory, otherwise the create command fails. When nil or blank string is set, the code is generated automatically using the uuEE's universe ID.
      # @option uuee [String] :description The description of the uuEE.
      # @option uuee [String, UU::OS::UESURI] :competent_role_uri UESURI of the competent role. When nil is set, the executive role from authorization is used. If no role is authorized to create the uuEE into specified container, the create command fails. If the selected role doesn’t have connected the uuEE interface with the selected meta artifact, the create command fails. If the selected role doesn’t belong to the same organizational unit as the new uuEE or if it doesn’t belong to one of superior organizational units, the create command fails.
      # @option uuee [UU::OS::Artifact::SecurityLevel] :security_level Security level of the new uuEE. When nil is set, security level is taken from the meta artifact. If the meta artifact is higher than the user’s security clearance, the create command fails.
      # @option uuee [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the new uuEE. When nil is set, the value NO_CONFIDENTIALITY is used.
      # @option uuee [Boolean] :searchable_in_territory Specifies if it is possible to search the uuEE within the territory. When nil is set, the default value from meta artifact is used.
      # @option uuee [String, UU::OS::UESURI] :icon_uri URI of the uuEE's icon. When nil is set or if nil UESURI ("ues:[-1]:[-1]:") is set, the uuEE uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option uuee [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the uuEE will be included among All roles from organizational unit. When nil is set, the value will be automatically set to <b>true</b>.
      # @option uuee [Boolean] :component_content Defines whether the primarily displayed content of the uuEE is component content. Default value is false.
      # @option uuee [String] :access_code1 Access Code 1 of newly created uuEE. Access code 1 must be 4-250 characters long and can only contain these characters: a-z, A-Z, 0-9, '.', '-', '_', '@'. If not specified, random code is generated.
      # @option uuee [String] :access_code2 Access Code 2 of newly created uuEE. Access code 2 must be 8-250 characters long and can only contain these characters: any character. If not specified, random code is generated.
      # @option uuee [String] :avatar_uri UESURI or URL of image which represents uuEE's avatar. If set to an empty string or nil, the uuEE will be created without avatar.
      # @option uuee [String] :universe_id Universe identifier of the user. When nil is set, universe identifier will be generated.
      #
      # @return [UU::OS::UESURI] UESURI of the new uuEE.
      #
      # @example
      #   # Create uuEE.
      #   UU::OS::UUEE.create('ues:UNI:FOLDER',
      #     :meta_artifact_uri=>'ues:UUEE_TERRITORY:UUEE',
      #     :app_code=>'CDS.GB',
      #     :code=>'6-6666-1',
      #     :description=>'',
      #     :first_name=>'uuEE',
      #     :surname=>'6-6666-1',
      #     :grant_all_from_organizational_unit=>false,
      #     :icon_uri=>'ues:[-1]:[-1]:',
      #     :searchable_in_territory=> false,
      #     :security_clearance=>'NO_CONFIDENTIALITY',
      #     :security_level=>'NO_CONFIDENTIALITY',
      #     :avatar_uri=>''
      #   )
      #
      def create(location_uri, uuee = nil)
        svc = UU::OS::REST::RemoteClient.new(UUEE)
        payload = UU::OS::UUEE::UUEECreate.new(uuee).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Sets access codes of an uuEE. It is possible to change both access codes or just one of them. For security reasons, both current access codes must be entered.
      #
      # @param [String, UU::OS::UESURI] uuee_uri UESURI of an uuEE whose attributes will be modified.
      # @param [UUEESetAccessCodes] access_codes Object containing both current and new access codes of the uuEE.
      #
      # @return [UU::OS::UESURI] UESURI of the specified uuEE.
      #
      # @example
      #   # Changes both access codes.
      #   UU::OS::UUEE.set_access-codes("ues:MT.SMITH.JOHN:6-6666-1",
      #     :access_code1 => "accessCode1",
      #     :access_code2 => "accessCode2",
      #     :new_access_code1 => "newAccessCode1",
      #     :new_access_code2 => "newAccessCode2"
      #   )
      #
      # @example
      #   # Changes access code 2 only.
      #   UU::OS::UUEE.set_access-codes("ues:MT.SMITH.JOHN:6-6666-1",
      #     :access_code1 => "accessCode1",
      #     :access_code2 => "accessCode2",
      #     :new_access_code2 => "newAccessCode2"
      #   )
      #
      def set_access_codes(uuee_uri, access_codes = nil)
        svc = UU::OS::REST::RemoteClient.new(UUEE)
        payload = UU::OS::UUEE::UUEESetAccessCodes.new(access_codes).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAccessCodes', uuee_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

    end
  end
end
